#include <windows.h>
#include <GL/glut.h>
#include <iostream>
#include <X3DTK/kernel.h>
#include <X3DTK/simplex3dglscene.h>
#include "camera.h"

using namespace std;
using namespace X3DTK;

Camera camera;
BBox bbox;
SimpleX3DGLScene scene;
char szFile[260];       // buffer for file name
char *fileName;

void mouse(int button, int state, int x, int y)
{
  QMouseEvent e(x,y);
  if (state == GLUT_DOWN)
  {
    switch (button)
    {
    case GLUT_LEFT_BUTTON:
      camera.frame()->startAction(ROTATE);break;
    case GLUT_MIDDLE_BUTTON:
      camera.frame()->startAction(ZOOM);break;
    case GLUT_RIGHT_BUTTON:
      camera.frame()->startAction(TRANSLATE);break;
    }      
    camera.frame()->mousePressEvent(&e,&camera);
  }
  else
  {
    camera.frame()->mouseReleaseEvent(&e,&camera);
  }
  glutPostRedisplay();
}

void motion(int x, int y)
{
  QMouseEvent e(x,y);
  if (camera.isManipulated())
  {
    camera.frame()->mouseMoveEvent(&e, &camera);
  }
  glutPostRedisplay();
}

void reshape(int w, int h)
{
  glViewport( 0, 0, w, h );
  camera.setWindowWidthAndHeight(w,h);
  glutPostRedisplay();
}

void init(void)
{
#ifdef GL_RESCALE_NORMAL
  glEnable(GL_RESCALE_NORMAL);
#endif 

  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
  glColor3f(1.0f,1.0f,1.0f);
}

void display(void)
{
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  camera.loadProjectionMatrix();
  camera.loadModelViewMatrix();

  scene.draw();

  glutSwapBuffers();
}

void load()
{
  OPENFILENAME ofn;       // common dialog box structure
  HWND hwnd;              // owner window

  // Initialize OPENFILENAME
  ZeroMemory(&ofn, sizeof(ofn));
  ofn.lStructSize = sizeof(ofn);
  ofn.hwndOwner = hwnd;
  ofn.lpstrFile = szFile;
  ofn.lpstrFile[0] = '\0';
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFilter = "All\0*.*\0Text\0*.TXT\0";
  ofn.nFilterIndex = 1;
  ofn.lpstrFileTitle = NULL;
  ofn.nMaxFileTitle = 0;
  ofn.lpstrInitialDir = NULL;
  ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
 
  fileName = 0; 
  if (GetOpenFileName(&ofn)==TRUE) 
  {
    fileName = ofn.lpstrFile;
  }
  
  if (fileName != 0)
  {
	scene.release();
    scene.load(fileName, false);

	float x = scene.getBBoxMin().x;
    float y = scene.getBBoxMin().y;
	float z = scene.getBBoxMin().z;

    Vec min, max;
	min.setValue(x, y, z);

	x = scene.getBBoxMax().x;
    y = scene.getBBoxMax().y;
	z = scene.getBBoxMax().z;

    max.setValue(x, y, z);
  
    camera.setSceneBoundingBox(min, max);
	camera.showEntireScene();
  }
}

void keyboard(unsigned char key, int x, int y)
{
  switch (key)
    {
    case 27: 
	  scene.release(); 
	  exit(0);
    case 'r':
      camera.setRevolveAroundPointFromPixel(x,y);
      break;
	case 'l':
	  load();
    default:
      return;
    }
  glutPostRedisplay();
}

int main(int argc, char *argv[])
{
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowPosition(200, 200);
  glutInitWindowSize(800, 600);
  glutCreateWindow("X3DGlutViewer");

  glutMouseFunc(mouse);
  glutMotionFunc(motion);
  glutKeyboardFunc(keyboard);
  glutReshapeFunc(reshape);
  glutDisplayFunc(display);

  init();
  load();   

  glutSwapBuffers();
  glutMainLoop();

  return 0;
}
