/****************************************************************************

 This file is part of the QGLViewer library
 Copyright (C) 2002-2004  Gilles Debunne (Gilles.Debunne@imag.fr)
 Version 1.3.6 Release 1. Packaged on Thursday February 5, 2004.

 http://artis.imag.fr/Members/Gilles.Debunne/QGLViewer

 libQGLViewer is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 libQGLViewer is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with libQGLViewer; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#include "spinningFrame.h"

using namespace std;

/*! Constructor sets default values : isSpinning() is \p false and updateInterval() is set to 40. */
SpinningFrame::SpinningFrame()
  : isSpinning_(false), updateInterval_(40), rotatesAroundCenter_(false), frozenOrientation_(false)
{
}

/*! Update the Frame orientation (and possibly position), according to the SpinningFrame parameters.
  Emits spinned(). */
void SpinningFrame::updateFrame()
{
  if (rotatesAroundCenter())
    if (orientationIsFrozen())
      setPosition(Quaternion(inverseTransformOf(spinningQuaternion().axis()), spinningQuaternion().angle()).
		  rotate(position()-rotationCenter())+rotationCenter());
    else
      rotateAroundPoint(spinningQuaternion(), rotationCenter());
  else
    rotate(spinningQuaternion());
  
}


/*! Starts the spinning of the frame. Use stopSpinning() to stop.
  
\p msec milliseconds will separate each update of the SpinningFrame position and orientation.
When no interval is specified, the previous updateInterval() is used, otherwise updateInterval()
is set to \p msecs.

See the <a href="../examples/spinningFrame.html">spinningFrame</a> example for an illustration. */
void SpinningFrame::startSpinning(const int msecs)
{
  if (msecs >= 0)
    setUpdateInterval(msecs);
  isSpinning_ = true;
}


